unit Demo8Settings;
//  If you are using Delphi 6 Enterprise then disable this compiler directive :
{$DEFINE MyXML}

interface

uses
  PtSettings, Classes;

type
  TElem = class(TSettingsGroup)
  private
    fIntVal: integer;
    fStrVal: string;
  published
    property  IntVal  : integer read fIntVal  write fIntVal;
    property  StrVal  : string  read fStrVal  write fStrVal;
  end;

  TIntIndexSettings = class(TSettingsGroup)
  private
    fCount: integer;
  protected
    function  ElemName(const i: integer): string;
    procedure Loaded(Storage: ISettingsStorage); override;
    procedure Saving(Storage: ISettingsStorage); override;
  public
    function  AddElem: TElem;
    procedure DelElem(const Elem: TElem);
    function  GetElem(const i: integer): TElem;
  published
    property  Count : integer read fCount write fCount;
  end;

  TStrIndexSettings = class(TSettingsGroup)
  protected
    procedure Loaded(Storage: ISettingsStorage); override;
  public
    function  AddElem(const ElemName: string): TElem;
    procedure RemElem(const ElemName: string);
    function  GetElem(const ElemName: string): TElem;
  end;

  TSettings = class(TSettingsGroup)
  private
    fIntIndex: TIntIndexSettings;
    fStrIndex: TStrIndexSettings;
  public
    constructor Create(const AName: string; const AParent: TSettingsGroup = nil);
    property  IntIndex: TIntIndexSettings read fIntIndex;
    property  StrIndex: TStrIndexSettings read fStrIndex;
  end;

var
  Settings: TSettings;
  Storage: ISettingsStorage;

procedure Initialize;
procedure Finalize;

implementation

uses
{$IFDEF MyXML}
  PtSettingsMyXML, MyXML,
{$ELSE}
  PtSettingsXML, XMLDoc,
{$ENDIF}
//  PtSettingsIni, PtSettingsReg,
  Demos, SysUtils, Windows;

procedure Initialize;
begin
  Settings := TSettings.Create( 'Demo8', nil);
  Storage := TXMLSettingsStorage.Create( NewXMLDocument, FileName('.xml') );
//  Storage := TIniSettingsStorage.Create(FileName('.ini'));
//  Storage := TRegSettingsStorage.Create( HKEY_LOCAL_MACHINE, '\SOFTWARE\PtSettings' );
  Storage.LoadAll( Settings );
end;

procedure Finalize;
begin
  Storage.SaveAll( Settings );
  Settings.Free;
  //Storage.Free;   The Storage variable is an interface
end;

{ TIntIndexSettings }

function TIntIndexSettings.AddElem: TElem;
begin
  Result  := TElem.Create( '', Self );
  inc( fCount );
end;

procedure TIntIndexSettings.DelElem(const Elem: TElem);
begin
  if  Count = 0 then
    raise Exception.Create(GroupPath+' has no Elems to delete');
  Storage.DeleteAll( Elem );
  RemoveSubGroup( Elem );
  dec( fCount )
end;

function TIntIndexSettings.ElemName(const i: integer): string;
begin
  Result  := 'Group_'+IntToStr( i )
end;

function TIntIndexSettings.GetElem(const i: integer): TElem;
begin
  Result  := SubGroups[i] as TElem;
end;

procedure TIntIndexSettings.Loaded(Storage: ISettingsStorage);
var
  i : integer;
begin
  inherited;
  //  Delete current Elem objects
  ClearSubGroups;
  //  Read in each Elem from Storage
  for i := 0 to Count-1 do
    Storage.LoadAll( TElem.Create( ElemName(i), Self ) );
end;

procedure TIntIndexSettings.Saving(Storage: ISettingsStorage);
var
  i : integer;
begin
  inherited;
  //  Give each Elem a unique name (so they do not write over each others data)
  for i := 0 to SubGroupCount-1 do
    SubGroups[i].GroupName  := ElemName(i);
end;

{ TStrIndexSettings }

function TStrIndexSettings.AddElem(const ElemName: string): TElem;
begin
  if  SubGroupByName( ElemName ) = nil  then
    Result  := TElem.Create( ElemName, Self )
  else
    raise Exception.Create('Elem '''+ElemName+''' already exists in '+GroupPath);
end;

function TStrIndexSettings.GetElem(const ElemName: string): TElem;
begin
  Result  := TElem( SubGroupByName( ElemName ) );
  if  (Result = nil) or not (Result is TElem) then
    raise Exception.Create('Elem '''+ElemName+''' in '+GroupPath);
end;

procedure TStrIndexSettings.Loaded(Storage: ISettingsStorage);
var
  Store : ISettingsStorageQuery;
  GroupNames  : TStringList;
  i : integer;
begin
  inherited;
{$IFDEF VER120} //  If Delphi 4 then use...
  Store := Storage as ISettingsStorageQuery;
{$ELSE}         //  otherwise use...
  if  Supports( Storage, ISettingsStorageQuery, Store ) then
{$ENDIF}
    begin
      GroupNames  := TStringList.Create;
      Store.GetGroupNames( GroupNames );  //  Get the names of the sub groups in the storage
      for i := 0 to GroupNames.Count-1  do
        if  SubGroupByName( GroupNames[i] ) = nil then  //  Don't load same group more than once
          Storage.LoadAll( TElem.Create( GroupNames[i], Self ) );
      GroupNames.Free;
    end;
end;

procedure TStrIndexSettings.RemElem(const ElemName: string);
var
  Elem  : TSettingsGroup;
begin
  Elem  := SubGroupByName( ElemName );
  if  Elem <> nil then
    begin
      Storage.DeleteAll( Elem );
      RemoveSubGroup( Elem );
    end;
end;

{ TSettings }

constructor TSettings.Create(const AName: string; const AParent: TSettingsGroup);
begin
  inherited;
  fIntIndex := TIntIndexSettings.Create('IntIndex', Self);
  fStrIndex := TStrIndexSettings.Create('StrIndex', Self);
end;

end.

